###################################################
# GS Algorithm 설명
###################################################
# 함수 gs 사용 방법: 
# 함수 gs는 남성(proposer)의 선호와 여성(receiver)의 선호를
# 인수로 하여 GS Algorithm을 이용한 남녀 매칭을 출력한다.

# 먼저 proposer, receiver의 선호를 집단별로 list형에 입력한다.
# 예를 들어 남성이 구혼할 때는 proposer로
# list(c(2, 3, 1), c(1, 2, 3), c(3, 1, 2))를 입력한다.
# 이는
# 남성1의 선호는 여2>여3>여1
# 남성2의 선호는 여1>여2>여3
# 남성3의 선호는 여3>여1>여2
# 임을 의미한다.
# 마찬가지로 receiver는 남성에 대한 여성의 선호 리스트이다.

# 함수 내부에서 receiver의 선호를 id가 아닌 순위로 변환한다.
# 예를 들어 어떤 여성의 선호가 c(2,3,1)   남2>남3>남1일 때
# (남1)은 3번째, (남2)는 1번째, (남3)은 2번째라는 순위 벡터로 변환한다.
# 즉, c(2, 3, 1)] -> c(3, 1, 2)로 변환한다.
# receiver 쪽에서 상대를 유보할 것인가는 순위를 통해 결정하기 때문이다.

########################################
# gs 안의 지역 변수 설명
########################################
# prefb: 여성에 대한 남성의 선호. prefg: 남성에 대한 여성의 선호
# nextgirl: 프로포즈하는 순서를 기록. 각 남성이 가짐. 차일 때마다 1씩 증가
# tempboy: 가 매칭한 남성 리스트, 처음에는 n+1이 들어감
# gid: 남성이 프로포즈할 상대 여성 id
# b: for용 카운터. 남성 1~n에 대응
# s: 현재 프로포즈 중인 남성 id
# t: 차인 남성 id. 이 값을 s에 대입하면 '이번에 차인 남성 t'가 자동으로 다음 s가 된다.

##################################
# 함수 gs 정의
##################################
gs130<- function(proposer,receiver)
{ n<-length(proposer)
prefb = proposer #남성의 선호로 인수 proposer를 그대로 입력
z=receiver  #일단 여성의 선호 리스트를 별도의 z에 대입
for(j in 1:n){
	for(i in 1:n){  
		z[[j]][i]=which(receiver[[j]]==i)}#for j
}#for i
prefg =z #여성의 선호를 순위 표기함
print("Man's preference") 
print(paste0(matrix(prefb))) 
print(paste0("Woman's preference") )
print(paste0(matrix(receiver)) )
for(i in 1:n){
	prefg[[i]] = append(prefg[[i]], n + 1)
}#여성의 선호에 n+1을 추가
nextgirl = rep(1, n)  #개인별 프로포즈용 카운터. 1위부터 시작하여 2위, 3위 순서로 진행
tempboy = rep(n + 1, n)  # while 조건용 값
for(b in 1:n){
	s = b
	while(s != n + 1 ){ #s가 1, 2, 3인 동안 루프*)
		k <- nextgirl[s]
		gid =prefb[[s]][k]#k=1부터 시작
		if(prefg[[gid]][s] < prefg[[gid]][tempboy[gid]])#현재 상대의 순위<가 상대의 순위
		{	t = tempboy[gid]#가 상대를 버림
		print(c("t=",t))#처음은 t=n+1, 바뀌면 t=이전의 가 매칭 남성 ID 
		tempboy[gid] = s # 가 상대를 바꿈
		s = t  # 다음의 s(proposer)를 지정함
		} #if 조건이 참이라면 실행문 종료
		else{#만약 s가 거부되면 nextgirl[s]에 1 더함
		nextgirl[s] <- nextgirl[s]+1
		}
	} # while 여기서 종료
}# for 여기서 종료

for(i in 1:n)#결과 표시
{print(paste0("Man",tempboy[i], " and Woman",i))}
}#gs 여기서 종료

###### random list generator
###### lg(n)은 길이 n의 무작위 리스트를 n개 반환
lg <- function(n){
	x <- list()
	for(i in 1:n){
		x[[i]] <- sample(1:n)
	}
	return(x)
}


#############################################
# 계산 예
#############################################
for(i in 1:5){
	gs130(lg(3),lg(3))}


for(i in 1:5){
	gs130(lg(4),lg(4))}
